% This function computes the matching between the sets of histograms psi1
% and psi2 by using the four different metrics.
% The output is the number of correct matches based on each metric.

function [numCorMatchProp, numCorMatchL1, numCorMatchCosine, numCorMatchDot]=calculateMatchAllMetrics(psi1, psi2)

%% computing the weight matrix based on the four metrics
% computing the weight matrix based on the proposed metric
weightMatProp =  weightMatCalcPropMetric(psi1, psi2);

% computing the weight matrix based on the three other heuristic metrics
[weightMatDot, weightMatCosine, weightMatL1] = weightMatCalcHeurMetrics(psi1, psi2);


%% performing the weighted bipartite matching
[~, s1Prop, s2Prop] = bipartite_matching(-weightMatProp+max(max(weightMatProp)));
[~, s1Dot, s2Dot] = bipartite_matching(weightMatDot);
[~, s1Cos, s2Cos] = bipartite_matching(-weightMatCosine+max(max(weightMatCosine)));
[~, s1L1, s2L1] = bipartite_matching(weightMatL1);


%% computing the accuracies
numCorMatchProp = sum(s1Prop==s2Prop);
numCorMatchL1 = sum(s1L1==s2L1);
numCorMatchCosine = sum(s1Cos==s2Cos);
numCorMatchDot = sum(s1Dot==s2Dot);
