% This function loads the histograms of the WBH dataset and computes the
% matching between all the users by using four distance metrics.
% The function also plots the popularity of different websites.
% The results are discussed in Section IV-B2 and Figure 8 of [1]
%
% [1]: "Where you are is who you are: User identification by matching statistics",
% Farid M. Naini, Jayakrishnan Unnikrishnan, Patrick Thiran, and Martin
% Vetterli, IEEE Transactions on Information Forensics and Security, 2015


clear all;
close all;
clc;


% loading the histograms
load WBH1;

%% computing popularities
locPop=sum((hist1+hist2),1);

% plotting the popularities
figure, loglog(sort(locPop,'descend'), 'linewidth', 2);
xlabel('Website identifier (favicon id)');
ylabel('Number of visits');


%% performing the matching between the histograms using four metrics
[numCorMatch_prop, numCorMatch_l1, numCorMatch_cosine, numCorMatch_dot] = calculateMatchAllMetrics(hist1, hist2);

% desplaying the results
fprintf(['Histogram matching for the WBH dataset:\n', ...
         'Number of users is: %d, number of locations is: %d \n', ...
         'The obtained accuracies are as follows:\n'], N, K);
fprintf(['Proposed metric: %1.2f%% \nl1-norm: %1.2f%%\nCosine distance: %1.2f%%\n', ...
        'Dot product: %1.2f%%\n'], numCorMatch_prop/N*100, numCorMatch_l1/N*100, ...
        numCorMatch_cosine/N*100, numCorMatch_dot/N*100);


    
