% This function loads the histograms of the WBH dataset and varies the
% number of considered websites and computes the
% matching between all the users by using four distance metrics.
% The results are shown in Figure 9 of [1]
%
% [1]: "Where you are is who you are: User identification by matching statistics",
% Farid M. Naini, Jayakrishnan Unnikrishnan, Patrick Thiran, and Martin
% Vetterli, IEEE Transactions on Information Forensics and Security, 2015



clear all;
close all;
clc;


% loading the histograms
load WBH1;

%% computing subset of users that visit the top five locations 
locPop = sum((hist1+hist2),1);
[~, locPopInd] = sort(locPop, 'descend');
top5Locs = locPopInd(1:5);

usersSub = (sum(hist1(: , top5Locs),2) > 0) & (sum(hist2(: , top5Locs),2) > 0); 

hist1 = hist1(usersSub, :);
hist2 = hist2(usersSub, :);

[N, K] = size(hist1);


%% choosing a subset of the websites and compute the matching 
KSet=[5:1:100, 105:5:200, 220:20:500, 550:50:1000, 1200:200:5000, 5500:500:1000, 11000:1000:82000, 83219];
matchRes=zeros(length(KSet),4);

for KIter=1:length(KSet)
    fprintf('Performing iteration %d from %d\n', KIter, length(KSet));
    
    % choosing the number of websites to consider
    KVal=KSet(KIter);
    chosenLoc=locPopInd(1:KVal);
    
    % computing the histogram on the chosen subset of websites
    hist1Subset=hist1(:, chosenLoc);
    hist2Subset=hist2(:, chosenLoc);
    
    % normalizing the histograms
    hist1Subset = myHistNorm(hist1Subset);
    hist2Subset = myHistNorm(hist2Subset);
    
    
    
    % computing weight matrix
    [numCorMatchProp, numCorMatchL1, numCorMatchCosine, numCorMatchDot]=calculateMatchAllMetrics(hist1Subset,  hist2Subset);
    
    matchRes(KIter,:)=[numCorMatchProp, numCorMatchL1, numCorMatchCosine, numCorMatchDot];
end    

%% plotting the results
figure
semilogx(KSet, matchRes/N*100, 'linewidth',2);
xlabel('Number of websites (K)');
ylabel('Percentage accuracy');
legend('Proposed metric', 'l_1-norm', 'Cosine distance', 'Dot product', 'Location', 'NorthWest');
title('Matching accuracy on the WBH dataset');