% This function is used in mypartialMatching function. It is written based on the
% theroy of Matroids (see Chapter 8 of [1]). It gets an independent set I 
% and weight vectors c1 and c2 and updates I.
% [1]: "Combinatorial Optimization", W. Cook, W. Cunningham, W. Pulleyblank,
% and A. Schrijver, Wiley Series in Discrete Mathematics and Optimization,
% Wiley, 2011.

function [Inew, c1Vec, c2Vec]=matroidIntsecUpdater(I, c1Vec, c2Vec)


N=sqrt(size(c1Vec,1)); %num users
k=size(I,1);

if k==0
    % all matrices are empty
    matchedV=[];
    matchedU=[];
    edgeIDs=[];
else
    matchedV=I(:,1);    % indices of matched nodes of set V
    matchedU=I(:,2);    % indices of matched nodes of set U
    edgeIDs=I(:,1)+(I(:,2)-1)*N;    %IDs of the edges in the current matching
end


%% computing S and T
availableNodesV=setdiff((1:N)', matchedV);
S=repmat(availableNodesV,[1 N])+N*repmat((0:N-1), [length(availableNodesV), 1]);
S=S(:);

availableNodesU=setdiff((1:N)', matchedU);
T=repmat(1+(availableNodesU'-1)*N,[N 1])+repmat((0:N-1)', [1, length(availableNodesU)]);
T=T(:);

m1=max(c1Vec(S));
m2=max(c2Vec(T));

if isempty(S) || isempty(T)
    % we are done!
    Inew=I;
    return;
end

%% constructing G
% node s is node N^2+2 and node r is node N^2+1
% first set of edges from nodes in S to node s
edgesSet1=[S, ones(length(S),1)*(N^2+2)];
edgesSetWeigth1=m1-c1Vec(S);

% second set of edges from node r to nodes in T
edgesSet2=[ones(length(T),1)*(N^2+1), T];
edgesSetWeigth2=m2-c2Vec(T);

% third set of edges 
edgesSet3=zeros(k*(N-1), 2);
edgesSetWeigth3=zeros(k*(N-1), 1);


if k~=0
    temp1=repmat(N*(0:N-1)', [1, k]);
    temp1=repmat(I(:,1)', [N, 1])+temp1;
    temp2=temp1(:);
    temp2(ismember(temp2, edgeIDs))=[];
    edgesSet3(:,1)=temp2;
    temp3=repmat(edgeIDs', [N-1, 1]);
    edgesSet3(:,2)=temp3(:); 
    edgesSetWeigth3(:)=c1Vec(temp3(:))-c1Vec(temp2);
end

% forth set of edges 
edgesSet4=zeros(k*(N-1), 2);
edgesSetWeigth4=zeros(k*(N-1), 1);


if k~=0
    temp4=repmat((1:N)', [1, k]);
    temp4=repmat(N*(I(:,2)'-1), [N, 1])+temp4;
    temp5=temp4(:);
    temp5(ismember(temp4, edgeIDs))=[];
    edgesSet4(:,2)=temp5;
    edgesSet4(:,1)=temp3(:); 
    edgesSetWeigth4(:)=c2Vec(temp3(:))-c2Vec(temp5);
end
 
% all edsges and weights 
allEdges=[edgesSet1; edgesSet2; edgesSet3; edgesSet4];
allEdgesWeights=[edgesSetWeigth1; edgesSetWeigth2; edgesSetWeigth3; edgesSetWeigth4];
[~, edgeSortIndice]=sort(allEdges(:,1)+(allEdges(:,2)-1)*(N^2+2), 'ascend');
allEdges=allEdges(edgeSortIndice, :);
allEdgesWeights=allEdgesWeights(edgeSortIndice, :);
G=sparse(allEdges(:,1), allEdges(:,2), 1, N^2+2, N^2+2, size(allEdges,1));

%% finding the set of recheable nodes from r and upadting the cost vectors
[dist]=graphshortestpath(G, N^2+1, 'Directed', true, 'Method', 'Dijkstra', 'Weights', allEdgesWeights);
R=(dist(1:end-2)<Inf);
ds=dist(end);
sigmaR=min(dist(R), ds);
% updating the cost vectors
c1Vec(R)=c1Vec(R)-sigmaR';
c2Vec(R)=c2Vec(R)+sigmaR';
    
%% reconstructing matrix G
m1=max(c1Vec(S));
m2=max(c2Vec(T));

% second set of edges 
edgesSetWeigthUpd1=m1-c1Vec(S);

% second set of edges 
edgesSetWeigthUpd2=m2-c2Vec(T);

% third set of edges 
edgesSetWeigthUpd3=zeros(k*(N-1), 1);
if k~=0
    edgesSetWeigthUpd3(:)=c1Vec(temp3(:))-c1Vec(temp2);
end

% forth set of edges 
edgesSetWeigthUpd4=zeros(k*(N-1), 1);
if k~=0
    edgesSetWeigthUpd4(:)=c2Vec(temp3(:))-c2Vec(temp5);
end

allEdgesWeightsUpd=[edgesSetWeigthUpd1; edgesSetWeigthUpd2; edgesSetWeigthUpd3; edgesSetWeigthUpd4];
allEdgesWeightsUpd=allEdgesWeightsUpd(edgeSortIndice);
[dist, path]=graphshortestpath(G, N^2+1, N^2+2, 'Directed', true, 'Method', 'Dijkstra', 'Weights', allEdgesWeightsUpd);

%% updating the path
if dist<Inf
    P=path(2:end-1);
    
    newEdgeIDs=union( setdiff(edgeIDs, P), setdiff(P, edgeIDs));
    Inew=zeros(length(newEdgeIDs), 2);
    Inew(:,1)=1+mod(newEdgeIDs-1, N); 
    Inew(:,2)=ceil(newEdgeIDs/N); 
    [~, tempIndice]=sort(Inew(:,1), 'ascend');
    Inew=Inew(tempIndice, :);
end
    