% This function performs maximum-weight bipartite matching with
% cardinality r. It is written based on the theroy of Matroids (see Chapter 8 of [1])
% and can be used to solve (A2) in [2]. The function first computes the maximum-weight
% matching of cardinality 1, then 2, and so on until the cardinality
% reaches r.
%
% Input: WMat is an NxN weight matrix of a bipartite graph. 1 <= r <=N is
% the cardinality of the output matching.
% Output: mVal is the value of the maximum-weight bipartite matching with
% cardinality r. mSet1 and mSet2 indicate the matching set. prevMatchings
% stores all the previous computed matchings with smaller sizes. 
%
% [1]: "Combinatorial Optimization", W. Cook, W. Cunningham, W. Pulleyblank,
% and A. Schrijver, Wiley Series in Discrete Mathematics and Optimization,
% Wiley, 2011.
% [2]: "Where you are is who you are: User identification by matching statistics",
% Farid M. Naini, Jayakrishnan Unnikrishnan, Patrick Thiran, and Martin
% Vetterli, IEEE Transactions on Information Forensics and Security, 2015



function [mVal, mSet1, mSet2, prevMatchings]=mypartialMatching(WMat, r)


% buffers
prevMatchings=cell(r,2);

Iold=[];
c1Vec=WMat(:);
c2Vec=zeros(size(c1Vec));

for matchingSize=1:r
        
    fprintf('Performing matching with cardinality %d\n', matchingSize);
    
    [Inew, c1Vec, c2Vec]=matroidIntsecUpdater(Iold, c1Vec, c2Vec);
    Iold=Inew;
    
    % storing previous matchings
    prevMatchings{matchingSize,1}=[Inew(:,1), Inew(:,2)];
    prevMatchings{matchingSize,2}=sum(WMat(Inew(:,1)+(Inew(:,2)-1)*size(WMat,1)));
        
end

% setting the output
mSet1=Inew(:,1);
mSet2=Inew(:,2);
mVal=sum(WMat(mSet1+(mSet2-1)*size(WMat,1)));