% this function generates histograms based on i.i.d. traces and computes
% the matching between the histograms using the four different metrics
% discussed in [1]
%
% [1]: "Where you are is who you are: User identification by matching statistics",
% Farid M. Naini, Jayakrishnan Unnikrishnan, Patrick Thiran, and Martin
% Vetterli, IEEE Transactions on Information Forensics and Security, 2015


clear all
close all
clc



%% parameters
N=100;  % number of users
K=10;   % number of locations
T=500;  % number of iid samples


%% generating histograms from iid traces

% buffers
hist1 = zeros(N,K);
hist2 = zeros(N,K);

% base probability law of users
pr0 = [1:K];

for userIter = 1:N
    
    % generating probability law of the current user
    pr = pr0 + randn(1,K);
    pr = abs(pr)/sum(abs(pr));
    
    % generating hostograms
    hist1(userIter,:) = mnrnd(T, pr)/T;
    hist2(userIter,:) = mnrnd(T, pr)/T;
end

%% performing the matching and displaying the results
[numCorMatchProp, numCorMatchL1, numCorMatchCosine, numCorMatchDot]=calculateMatchAllMetrics(hist1, hist2);

fprintf(['Histogram matching from iid simulated data:\n', ...
         'Number N of users is: %d, number K of locations is: %d \n', ...
         'Number T of iid samples is: %d \n',... 
         'The obtained accuracies are as follows:\n'], N, K, T);
fprintf(['Proposed metric: %1.2f%% \nl1-norm: %1.2f%%\nCosine distance: %1.2f%%\n', ...
        'Dot product: %1.2f%%\n'], numCorMatchProp/N*100, numCorMatchL1/N*100, ...
        numCorMatchCosine/N*100, numCorMatchDot/N*100);