% This function computes the weight matrix between the sets of histograms
% psi1 and psi2, by using the dot product, the L1 norm, and the cosine
% distance.


function [weightMatDot, weightMatCosine, weightMatL1] = weightMatCalcHeurMetrics(psi1, psi2)

[N, K] = size(psi1);

% making sure that each row of psi1 and psi2 sums to one
psi1= myHistNorm(psi1);
psi2= myHistNorm(psi2);


%% computing the weight matrix based on the inner product
weightMatDot = sparse(psi1*psi2');


%% computing the weight matrix based on the cosine distance
% Cosine similarity weight Matrix
psi1_L2 = full(sqrt(sum(psi1.^2,2)));
psi2_L2 = full(sqrt(sum(psi2.^2,2)));

[i, j, val] = find(weightMatDot);
weightMatCosine = sparse(i, j,  val./(psi1_L2(i).*psi2_L2(j)), N, N);


%% computing the weight matrix based on L1 norm
valL1 = - (sum(abs(psi1(i,:)-psi2(j,:)),2) - 2);
weightMatL1 = sparse(i, j, valL1, N, N);



weightMatDot = full(weightMatDot);
weightMatCosine = 1 - full(weightMatCosine);
weightMatL1 = full(weightMatL1);